# Introduction to R Day 1: Input, Output and Data types

# This is an empty template that you will fill in during the class, where there is an * it means there is something to do! When you use # in an R script it marks it as annotation not script, so you can write as many notes as you like as long as they are denoted with a #.

# Learning R is a steep learning curve, it is just like a new language you have to learn its syntax, so you will have to give it time and practice!

# My aim with this class is to give you the tools to be able to use R to analyze your own datasets. This means you have to understand the basic structures in R as you will inevitably come across errors and you have to be able to figure out what is causing them. Today were are focusing on basic data formats as one common source of errors is formating - the function expects a dataframe and you gave it a vector or vice versa. 


#==General data management & reading text file into R==

## General data management: Rules for naming files, folders and columns - NEVER use spaces or complex characters. If you want a gap use an underscore e.g. marsupialmass_continent.txt, this makes it much easier for computer programs to read. 

##* Open R and set your working directory to your Intro2R folder: Misc -> Change Working directory this saves typing the path to the file each time.


## Data files from excel to R: .txt, .csv


##* Open marsiupialmass_continent.txt and look at it


##* Read in marsiupialmass_continent.txt using the read.table function. You need to name the dataset - think of a suitable name so you know what it is but also short and easy to type! You assign the data to the name using an arrow <- (can also use = but don't as it can get confusing). You then need to tell R where to find the dataset, we set the working directory to be the Intro2R folder so all you need to do is tell it the name of the file and put it in quotation marks to denote it is a string i.e. a sequence of characters that it needs to read.

marmass<-read.table("marsupialmass_continent.txt")


##* Look at the dataset in R by typing the name you assigned it to - are there any problems? Yes! it is recognizing the column names as the first row of our data

marmass

## You can tell the read.table function that the dataset contains column names by using header=T, each command is separated by a comma. Also, do you want the species names to be the row names rather than a data column? If so, tell it which column contains the names using row.names=1). 

marmass<-read.table("marsupialmass_continent.txt", header=T) # now our column names are read in as headers not as a row of data.


## Understanding the structure of the data - useful to tell if it has been read in correctly


##* find the dimensions of the dataset - how many rows and columns using the dim function

dim(marmass) # this gives us the number of rows followed by the number of columns

##* using class() find out what data type R has stored our dataset as  

class(marmass) # it is a dataframe 

#==Dataframes in R==

##These are the most common and arguably the most useful data structure in R. Lets explore them

##* Look at the structure of the marsupial dataset we read into R - this can tell us a lot about the dataset and what can and can't be done to it.

str(marmass) # this gives us basic information about the dataset, its class, dimensions and about each column whether it is a factor or numeric etc. 

## We can perform various mathematical operations on numeric data in dataframes

##* Multiply log mass by 2 and add it to a new column in the dataframe. 

marmass$continent # this accesses the continent column 
marmass[,2] # this accesses the continent column too as it is the second column
marmass$doublemass<-marmass$log_mass*2 # here we create a new column called double mass in the marmass dataframe and adds to this new column log_mass multiplied by 2.

## We can also manipulate the dataframe removing rows or columns. Look at the dimensions of the dataframe using the function dim(), you can see we have a 124*4 dataframe, that is 124 rows and 4 columns - the rows always come first.

##* Extract the first row of the dataframe, then the first column

marmass[1,]

##* extract the 3rd column of the 3rd row

marmass[3,3]

##* Remove Sminthopsis_psammophila from the dataset 
# first identify its row
newmarmass<-marmass[-114,] # create a new object and assign the marmass dataframe to it minus the 114th column..

##* Remove that fourth column that you just added

newmarmass<-newmarmass[,-4] # 


##* Save the dataframe using write.table

write.table(newmarmass, file="newmarsupial.txt")

## Note: Brackets [] versus parentheses () in R. The distinction is important: parentheses contain all the information that you need to pass to a function or add to an object while brackets are for indicating the position of item in an object.


#==Vectors in R==

# Vectors contain a single string of numbers or characters

##* Extract mass and add it to a new object (vector) 

logmass<-marmass$log_mass


##* Make sure we know which mass belongs to which species by adding names 

names(logmass)<-marmass$Genus_species

# dim() doesn't work a vector has a single dimension, to see how many elements it has type:

length(logmass)

##* Create your own vector and assign a character string to it, then try a string of numbers

foo<-10

## When we have multiple numbers or characters  we need to tell R to concatenate them. 

foo<-c(10, 20, 31)

##* Subset the vector - extract the 2nd element from the vector you just created. 

foo[2]

##* Mathematical operations also work on vectors - generate 2 vectors each with 5 elements and multiply them together

foo1<-c(5,7,9,13,17)

foo2<-c(1,2,3,4,5)

multifoo<-foo1*foo2

##* Now combine the 2 vectors into a single vector

foo3<-c(foo1, foo2)

##* We can also create dataframes out of vectors using the data.frame function - do this with the 2 vectors we just generated - try adding row.names too.

foodat<-data.frame(foo1, foo2)

#==Matrices in R==

## Matrices are very similar to dataframes but there are a few crucial differences - lets figure out what they are. 

##* Convert your marsupial mass data.frame into a matrix using the function as.matrix but assign it to a new name. Look at it and its structure - does anything look different?

marmat<-as.matrix(marmass)

##* Try repeating what we just did with the dataframe: multiply log mass by 2 and add it to a new column in the dataframe. 

marmat$doublemass<-marmat$log_mass*2 # can't use $ with a matrix!

marmat[,4]<-as.numeric(marmat[,3])*2

str(marmat)

##* We have learnt two important things about matrices 1) we can't use $ to access columns 2) matrices need data to be of the same type - can't combine numeric and factor data otherwise all will be recognized as factors.


#==Lists in R==

## Lists are a little different as they contain multiple objects which can be of different types and lengths - including dataframes and vectors. We have already seen a list as a matrix is a special kind of list with a particular set of attributes. You will encounter lists when looking at the output of various statistical functions. 

##* Create a list with the 2 vectors you just created and the original marsupial dataframe and look at its structure

datlist<-list(marmass, foo1, foo2)

##* Extract the second element of the list which is foo1 - you need double brackets [[]]

datlist[[2]]

##* Now try extracting the mass column from the dataframe in the 3rd element of the list

datlist[[1]][,3]

datlist[[1]]$log_mass

#=====EXERCISES=====

# These should be started in class and may be finished at home as they apply what we have covered in class to a new dataset.

#1) Read in the Carnivora Dataset - look at it first, notice this one has a lot of missing data denoted by NA you need to tell read.table to recognize NA as missing data using na.string=NA

#2) Put interbirth interval into its own vector and convert it into years as it is currently in days and make sure it is named. 

#3) Add the new interbirth interval in years as an additional column to the Carnivora Dataset  (or do 2 & 3 in a single operation)

#4 ) Remove Vulpes vulpes from the dataset, assigning the newly pruned dataset to a new object

#5) Save the new Carnivora dataset






