# Introduction to R Day 2: Plotting 

# R is an exceptionally powerful graphics tool but generating exquisite figures takes patience and knowledge of how R constructs a plot. Today we are going to focus on the basic plotting functions available in the base package of R using the plot function. However, there are several other packages such as ggplot, lattice and plotrix that can be very useful for more complex plots. Also don't forget plotting is an incredibly important step in data exploration before and during data analysis, it allows you to visualize the patterns within your data.

# This is an empty template that you will fill in during the class, where there is an * it means there is something to do! 

##* Open R and if you haven't already done so, set your working directory to your Intro2R folder. Read in the Carnivora Dataset setting the row names to the latin binomial in the first column.

carn<-read.table("CarnivoraDataset.txt", header=T, row.names=1)

#=====SIMPLE SCATTER PLOTS=====
## The most basic plot you can generate is a simple scatter plot

##* Plot body mass on the X axis and gestation length on the Y

plot(carn$AdultBodyMass_g, carn$GestationLen_d)

##* Now lets look at the arguments that we that we can pass to the default plot gives us by looking at the help file

?plot.default

## You can see the default plot for two simple continuous variables is a scatter plot with each element plotted as a point

## This plot can be customized in many ways, lets explore some of them

##* We can change the range of the axes using xlim and ylim arguments. Replot the data ignoring the 3 largest species

plot(carn$AdultBodyMass_g, carn$GestationLen_d, xlim=c(0,500000), ylim=c(0,250))


##* We can label the axes with more informative labels using xlab and ylab arguments. Replot the data with more informative axis labels.

plot(carn$AdultBodyMass_g, carn$GestationLen_d, xlim=c(0,500000), xlab="Body Mass", ylab="Gestation Length")

##* We can change the size of the point using the cex argument. Replot the data with larger points (the default is cex=1)

plot(carn$AdultBodyMass_g, carn$GestationLen_d, xlim=c(0,500000), xlab="Body Mass", ylab="Gestation Length", cex=1.5)

##* We can change the type of point using the pch argument, look at the Symbols and Line Types pdf in the Intro2R folder. Replot the data with filled triangles
plot(carn$AdultBodyMass_g, carn$GestationLen_d, xlim=c(0,500000), xlab="Body Mass", ylab="Gestation Length", cex=1.5, pch=17)

##* We can change the colour of the point using the col argument, look at the color chart pdf in the Intro2R folder. Replot the data using a different colour. 

plot(carn$AdultBodyMass_g, carn$GestationLen_d, xlim=c(0,500000), xlab="Body Mass", ylab="Gestation Length", cex=1.5, pch=17, col="darkolivegreen")

##* We can also colour each point a different colour using colour palettes in R: rainbow, topo.colors, heat.colors, terrain.colors - these all require you to pass them the number of colours you want to sample.

plot(carn$AdultBodyMass_g, carn$GestationLen_d, xlim=c(0,500000), xlab="Body Mass", ylab="Gestation Length", cex=1.5, pch=17, col=rainbow(length(carn$AdultBodyMass_g)))


##* Due a wide variability in mass these data will be easier to visualize using a log plot.  Replot the data with log mass on the X axis and log gestation length on the Y using the argument log="xy" .... Remember to change the axis labels. 

plot(carn$AdultBodyMass_g, carn$GestationLen_d, xlab="Body Mass", ylab="Gestation Length", cex=1.5, pch=17, log="xy")

plot(log(carn$AdultBodyMass_g), log(carn$GestationLen_d), xlab="Body Mass", ylab="Gestation Length", cex=1.5, pch=17,)


## Oftentimes we want to plot several groups to compare the differences - we are going to plot cats (Felidae) and dogs (Canidae): mass vs gestation length. Next week we will learn about clever ways of subsetting data but today we are going use a method we already know. Identify the rows that belong to cats and plot the log mass vs log gestation length .

##* Now identify the the rows that belong to canids and plot log mass vs log gestation as points - this is how we add data to an existing scatterplot - make sure to use a different colour.

plot(log(carn$AdultBodyMass_g[2:36]), log(carn$GestationLen_d[2:36]), xlab="Body Mass", ylab="Gestation Length", cex=1.5, pch=17)
 
 points(log(carn$AdultBodyMass_g[45:84]), log(carn$GestationLen_d[45:84]), cex=1.5, pch=17, col="red")
 
 ##* Now we can add a legend to the plot using the legend function. You first give it the x,y co-ordinates telling it where to place the legend or you can give it:  "topright", "topleft", "bottomright", "bottomleft". Then you provide the text for the legend and the type of point and colours. To remove the frame use the argument bty="n"

legend("bottomright", c("Canid", "Felid"), pch=17, col=c("black", "red"), bty="n")


#=====SIMPLE HISTOGRAMS=====
## A histogram is a plot of counts or proportions of data in discrete bins and is useful for visualizing the distribution of the data - does it look normally distributed ? etc.

##* First plot a histogram of geographic range area (GR_Area_km2) and then the log of geographic range area. 

hist(carn$GR_Area_km2)
hist(log(carn$GR_Area_km2))


##* You can add informative titles using the argument main= and axis labels using the same argument we just used for our scatterplot.

hist(log(carn$GR_Area_km2), xlab="Geographic Range", main="", col="blue")


##* How do you choose the number of intervals? Too few bins will obscure patterns within the data and too many will show too much noise within the data. To illustrate this first plot the log geographic range data using 3 breaks and then using 50. In reality the only way to choose the number of intervals is to play with breaks and pick one that shows the underlying pattern 

hist(log(carn$GR_Area_km2), xlab="Geographic Range", main="", col="blue")

##* We can add 2+ histograms to the same plot using the argument add=T, this is useful when trying to illustrate differences in the distribution between several different groups. Plot a histogram of the log of the first half of the data and add to that plot, the histogram of the log of the second half of the data. Colour each group differently. 

hist(log(carn$GR_Area_km2[1:143]), xlab="Geographic Range", main="", col="blue")
hist(log(carn$GR_Area_km2[144:286]), xlab="Geographic Range", main="", col="red", add=T, breaks=15)


## What do we need to do to make a better plot? a) We need to use the same bins for both groups b) Change the y axis so that the second group can be seen and c) ideally have the colours be transparent so we can see the two distributions.

hist(log(carn$GR_Area_km2[1:143]), xlab="Geographic Range", main="", col="blue")
hist(log(carn$GR_Area_km2[144:286]), xlab="Geographic Range", main="", col="red", add=T)


##* Through experimentation find the number of breaks that generates the same number of bins in the 2nd group as the first and the right y limits. 

hist(log(carn$GR_Area_km2[1:143]), xlab="Geographic Range", main="", col="blue")
hist(log(carn$GR_Area_km2[144:286]), xlab="Geographic Range", main="", col="red", add=T, breaks=15)

##* To get colours with a degree of transparency we can use the rgb colour specification function and define the alpha level: Semi-transparent colors (0 < alpha < 1). To get a semi-transparent red you use col=rgb(1,0,0, 0.5), read the arguments for rgb and figure out how to get semi-transparent blue and then plot the two histograms using these two colours.

hist(log(carn$GR_Area_km2[1:143]), xlab="Geographic Range", main="", col=rgb(0,0,1, 0.6))
hist(log(carn$GR_Area_km2[144:286]), xlab="Geographic Range", main="", col=rgb(1,0,0, 0.6), add=T, breaks=15)

##* Notice the default histogram plots the counts to get the relative frequency you need to use the argument freq=F. Replot your 2 groups using frequency - what is the maximum possible y value?

hist(log(carn$GR_Area_km2[1:143]), xlab="Geographic Range", main="", col=rgb(0,0,1, 0.6), freq=F)
hist(log(carn$GR_Area_km2[144:286]), xlab="Geographic Range", main="", col=rgb(1,0,0, 0.6), add=T, breaks=15, freq=F)


#=====ADDING ANNOTATIONS =====

##* We can add text to the margins using the mtext function, you have to tell it which side to add it to, the order is always: bottom (South), left side (West), top (North) and right side (East). Add a title to the top margin - you can change the line it is on using the line argument.

plot(carn$AdultBodyMass_g, carn$GestationLen_d, xlim=c(0,500000), xlab="Body Mass", ylab="Gestation Length", cex=1.5, pch=17,)
mtext("Carnivora Gestation Allometry",side=3, line=2, cex=1.2)

##* We can add text to the plot using the text function. For example if we wanted to label one of the points we can work out the x and y co-ordinates where we want to place our text. 

text(380000, 80, "Outlier species")

##* We can add lines to the plot using the segments function, you need to tell it the x-y coordinates of the start of the line and the end of the line. Draw a horizontal line at a gestation length of 200 days across the whole plot.

segments(0, 200, 5000000, 200)

#=====UNDERSTANDING THE PLOTTING WINDOW=====
## To really be able to take advantage of the powerful plotting functions in R you need to understand how R constructs a plot.

##* Plot a scatter plot of log mass vs log home range

plot(log(carn$AdultBodyMass_g), log(carn$HomeRange_km2), xlab="Body Mass", ylab="Home Range")

##* Now we are going to add coloured boxed to define the plotting area ("plot") and the figure ("figure") area using the box function. The space between these two is the margin area.

box("plot", col="blue")
box("figure", col="red")

## We can alter many aspects of the plot by calling the function par. We can change the margins using the mar argument () to see the current margins type par()$mar (Remember the order is always: bottom (South), left side (West), top (North) and right side (East)). 

##* Replot the scatter plot and the boxes after setting the margins so that the right side margin is much larger,

par(mar=c(5.1, 4.1, 4.1, 10))
plot(log(carn$AdultBodyMass_g), log(carn$HomeRange_km2), xlab="Body Mass", ylab="Home Range")

##* Oftentimes it is helpful to have more than one plot in the same window - we can set the number of rows and columns within the plot using par(mfrow=c(x,x)). Set up 4 plots in a 2x2 grid and plot log body mass vs log: gestation length, age at first birth, interbirth interval and longevity.

par(mfrow=c(2,2))

plot(log(carn$AdultBodyMass_g), log(carn$GestationLen_d), xlab="Body Mass", ylab="Gestation Length")
plot(log(carn$AdultBodyMass_g), log(carn$AgeatFirstBirth_d), xlab="Body Mass", ylab="Age at first birth")
plot(log(carn$AdultBodyMass_g), log(carn$InterbirthInterval_d), xlab="Body Mass", ylab="Interbirth Interval")
plot(log(carn$AdultBodyMass_g), log(carn$MaxLongevity_m), xlab="Body Mass", ylab="Maximum Longevity")

#=====EXERCISE=====#

## Read in the parrotfish dataset. Construct a publication-ready 2-panelled figure - this means suitably labelled axes and titles and where necessary legends. Plot 1: Display the distribution of gape distance in members of the Scarus genus vs all other parrotfishes (think about what type of plot should you use). Plot 2: Display the relationship between standard length and gape distance across parrotfishes using different colours for members of the Scarus genus and other parrotfishes (think about whether you want to transform the data) and label the smallest fish with its species name 

## If you are struggling with getting everything to look good and fit in the plotting window you can change the size by using quartz(width=x, height=x) on a mac or windows(width=x, height=x) on a pc.

parrot<-read.table("~/Desktop/Intro2R/parrotfishesRawMorphology.txt", header=T, row.names=1)

quartz(width=10, height=5)  # telling mac window to be a certain height and width use windows() on a pc.

par(mfrow=c(1,2))

# A histogram is a great way to look at the distribution of a single trait! so plot 2 histograms one of Scarus gape and one for the other parrotfishes
hist(parrot$gape_distance[c(1:11, 31:37)], col=rgb(0,0,1,0.7), ylim=c(0,15), main="Gape Distances across Parrotfishes", xlab="Gape Distance (mm)", cex=0.8)
hist(parrot$gape_distance[12:36], col=rgb(1,0,0,0.7), add=T)
legend("topright", c("Not Scarus", "Scarus"), pch=15, col=c(rgb(0,0,1,0.7), rgb(1,0,0,0.7)), bty="n")

plot(log(parrot$standard_length[c(1:11, 31:37)]), log(parrot$gape_distance[c(1:11, 31:37)]), xlab="Log Standard Length", ylab="Log Gape Distance", pch=16, col=rgb(0,0,1,0.7))
points(log(parrot$standard_length[12:36]), log(parrot$gape_distance[12:36]), pch=16, col=rgb(1,0,0,0.7))
mtext("Length and gape across Parrotfishes", side=3, line=1.6, font=2, cex=1.2)
text(4.05, 0.7, "Cryptotomus roseus", cex=0.7)
legend("topleft", c("Not Scarus", "Scarus"), pch=16, col=c(rgb(0,0,1,0.7), rgb(1,0,0,0.7)), bty="n")
