# Introduction to R Day 3: Dataset manipulation 

# You can do a lot of dataset manipulation in R including subsetting and merging. Many people ignore these functions and do it in excel however, it can streamline your dataset preparation and analyses if you do it in R (but it may give you a headache in return!)

# This is an empty template that you will fill in during the class, where there is an * it means there is something to do! 

##* Open R and if you haven't already done so, set your working directory to your Intro2R folder. Read in the Carnivora Dataset.

carn<-read.table("CarnivoraDataset.txt", header=T, row.names=1)


#==Introduction to logic statements and logic operators in R==
## It might seem unusual but we are going to take a slight detour into philosophy and in particular Logic. In logic we are interested in true and false statements. 
# == is "exactly equal" in R as = is equivalent to <-
1+1==2 # R returns TRUE (T)
1+3==2 # R returns FALSE (F)

## For more complex cases we need conectors or logic operators : NOT, AND, OR (as well as if_then or if_and_only_if but we won't discuss these)

# OR: | is the symbol for or in R
1+1== 2 | 2+2 == 2 # R returns TRUE (T)
1+3 == 2 | 2+2 == 2 # R returns FALSE (F)
# AND: & is the symbol for and in R
1+1== 2 & 2+2 == 2 # R returns FALSE (F)
1+1== 2 & 2/1 == 2 # R returns TRUE (T)
# NOT: ! is the symbol for not in R
1+2==2 # R returns FALSE (F)
!1+2==2 # R returns TRUE (T)

## Can also use greater than (>) or less than (<) too and don't forget if you want to include the number you are referencing it is equal to or greater than (>=) or equal to or less than (<=)

##* Create a vector x with numbers 1 through 5 and see if x is greater than 2 - what do you predict it will show?
x<-c(1:5)

#==Subsetting data using logical operators==
## Last week we used the clunky way of idenifying different row numbers to plot the canids and felids separately. Now I want to show you the syntax in R that allows you to do it programmatically using the [] to look within the dataset and logic statements. 

##* Put felid body mass data into a new vector using indices

felidmass<-carn$AdultBodyMass_g[carn$Family=="Felidae"] # go to the carn dataset find the adult mass column and give me all the values of mass where the column family is equal to Felidae

##* Now add the names

names(felidmass)<-row.names(carn)[carn$Family=="Felidae"]


##* Create a scatterplot of mustelid mass vs geographic range (GR_Area_km2)

plot(log(carn$AdultBodyMass_g[carn$Family=="Mustelidae"]), log(carn$GR_Area_km2[carn$Family=="Mustelidae"]))

## We can also subset the complete dataframe using logical statements and the subset function (can also use indices). 

##* Create a subset of the carnivore data that includes all canids for this we need to say find all rows corresponding to Canidae in the family column. 

caniddata1<-subset(carn, Family=="Canidae") # subset method
caniddata2<-carn[carn$Family=="Canidae",] #indexing method need the , to tell it to use rows and columns without it, it assumes it is a vector and it won't work generating the error "Error in `[.data.frame`(carn, carn$Family == "Canidae") : undefined columns selected"

##* Create a subset of the carnivore data that includes all canids and ursids - for this we need to say find all rows corresponding to Canidae OR Ursidae in the family column.

canursiddata<-subset(carn, Family=="Canidae" |  Family=="Ursidae")

##* Create a subset of the carnivore data that includes all carnivores that weigh more than a kilo - for this we need to say find all rows that have a weight greater than 1000g in the AdultBodyMass_g.

bigcarnivores<-subset(carn, AdultBodyMass_g > 1000 )

##* Create a subset of the carnivore data that includes all viverrids that weigh more than a kilo - for this we need to say find all rows that have a weight greater than 1000g in the AdultBodyMass_g AND corresponding to Viverridae in the family column.

bigviverrids<-subset(carn, Family=="Viverridae" & AdultBodyMass_g > 1000 )

##* Create a subset of the carnivore data that includes all carnivores except that are NOT (Otariidae OR Phocidae OR Odobenidae) i.e. terrestrial carnivores

terrestrialcarn<-subset(carn, !(Family=="Otariidae" |  Family=="Phocidae"  | Family=="Odobenidae") )


#==Merging dataframes==

##*  When you have identical columns in the exact same order (data) but no common rows (taxa) you want to bind the rows together using the function rbind. Bind the canid and big viverrid datasets together

newdat<-rbind(caniddata1 , bigviverrids) # 

## When you have two datasets with identical taxa (which are not necessarily in the same order) but diffferent columns you can merge the datasets using the function merge. 

##* Read in CarnivoraDataset2

carn2<-read.table("CarnivoraDataset2.txt", header=T)

##* Merge the two carnivore datasets using their latin binomial as the common column. The first dataset has the binomial as the row names and the second it is in the 2nd column.

carndiet<-merge(carn,carn2, by.x="row.names",  by.y=2)

## You can also use merge when they share overlapping taxa but do not have the same number of species. 

##* Create a subset of the second carnivoran dataset with just canids.
head(carn2)
caniddata2<-subset(carn2, Family=="Canidae")

##* Merge this new dataset with our original carnivore dataset

carndiet2<-merge(carn, caniddata2, by.x="row.names",  by.y=2 )

## What has happened? the default is to only merge the rows that are in both datasets, so we only have merged the canid data. But what if we want the whole carnivore dataset merged with the canid dietary data? We need to specify the argument all.x=TRUE, then extra rows will be added to the output, one for each row in x that has no matching row in y,these rows will have NAs in those columns that are usually filled with values from y.

carndiet2<-merge(carn, caniddata2, by.x="row.names",  by.y=2, all.x=TRUE )

#=====EXERCISES=====#
##* Read in the parrotfish dataset and the ScarusSparisoma fins dataset. 

parrot<-read.table("parrotfishesRawMorphology.txt", header=T, row.names=1)
scarus<-read.table("ScarusSparisomaFins.txt")

#1) Create a subset of the parrotfish dataset where every species is shorter than 250 in length with an adductor muscle mass of at least 4 or above

smallstrongparrot<-subset(parrot, standard_length <250 & adductor_mass>=3)

#2) Create a subset of the parrotfish data that includes the extremes of body mass either less than 15 or greater than 1000

extremeparrot<-subset(parrot, body_mass <15 | body_mass > 1000)

# 3) Create a scatterplot of standard length vs fin angle for the genera Scarus and Sparisoma separately. Remember the plot arguments you learnt (learned) last week to make it look pretty!

scardat<-merge(parrot, scarus, by.x="row.names", by.y="row.names")

plot(log(scardat$standard_length[scardat$genus=="Scarus"]), scardat$fin_angle[scardat$genus=="Scarus"], col="turquoise", pch=16, xlab="Log Standard Length", ylab="Fin Angle", xlim=c(4, 6)) # be careful you need to set up the x limits as several Sparisoma species are much smaller than any Scarus.

points(log(scardat$standard_length[scardat$genus=="Sparisoma"]), scardat$fin_angle[scardat$genus=="Sparisoma"], col="grey50", pch=16)