# Introduction to R Day 4: Basic Statistics

# R is a statistical computing environment, so it is incredibly powerful when it comes to implementing any kind of statistical analysis. Many of the basic statistical analyses are part of the base package in R but many others will require you to download and install separate R packages.  Today we are going to cover basic frequentist statistics (as apposed to Bayesian) where it is all about the p-value. 

# This is an empty template that you will fill in during the class, where there is an * it means there is something to do! 

##* Open R and if you haven't already done so, set your working directory to your Intro2R folder. Read in the Parrotfish and the Parrotfish Ecology Dataset and merge them into a single dataset

parrotmorph<-read.table("~/Desktop/Intro2R/parrotfishesRawMorphology.txt", header=T, row.names=1)
parrotecol<-read.table("~/Desktop/Intro2R/parrotfishecology.txt", header=T, row.names=1)

parrot<-merge(parrotmorph, parrotecol, by.x="row.names", by.y="row.names")


#===TESTING THE ASSUMPTIONS OF PARAMETRIC STATISTICS==
## What are the assumptions of parametric statistics?  1) We assume the data has come from a probability distribution and makes inferences about the parameters based on that distribution. For continuous characters (numeric not integer or factors) the assumed distribution is usually the normal distribution.

##* How might you look to see whether parrotfish mass and gape are normally distributed?

par(mfrow=c(1,2))
hist(parrot$body_mass)
hist(parrot$gape_distance)

##* If the data are not normally distributed you can transform the data so that they become more normal. For morphological data a common transformation is the log transform see if this works - you can use the natural log or log10 - which is better. 

par(mfrow=c(2,2))
hist(log10(parrot$body_mass))
hist(log10(parrot$gape_distance))
hist(log(parrot$body_mass))
hist(log(parrot$gape_distance))


#===BASIC PARAMETRIC STATISTICS===
## We are going to cover basic t-tests, correlation, linear regression, ANOVA. First lets remind ourselves about some basic statistics. All these tests rely on the p-value, which is the probability that the observed result was obtained by chance i.e. when the null hypothesis is true. If the p-value is lower than a preset alpha value we reject the null hypothesis - alpha is usually 0.05, so we have 95% certainty that our results reflect a genuine effect. We are now going to work through a bunch of questions picking the right test for each...

##* Do parrotfishes with intramandibular joints (imj) have different protrusion distances?  What test should we use?

##* Is the variance in the two groups equal?

var(log10(parrot$protrusion[parrot$imj==1]))
var(log10(parrot$protrusion[parrot$imj==0]))

##* Need to account for unequal variances using the var.equal=F argument. Also does our question require 2-tailed or 1-tailed test?

imjprot<-t.test(log10(parrot$protrusion[parrot$imj==1]), log10(parrot$protrusion[parrot$imj==0]), alternative="two.sided", var.equal=F)

# an alternative way of implementing exactly the same analysis uses a statistical formula. The formula takes the structure y~model where y is the reponse variable and the model defines the predictor variable(s).

imjprot<-t.test(log10(parrot$protrusion) ~ as.factor(parrot$imj), alternative="two.sided", var.equal=F) 

##* We can use this formula to generate a simple boxplot to visualize the data, dark line is the median the box represents the 1st and 3rd quartile and the whiskers extend to the smallest and largest non-outlier.

plot(log10(parrot$protrusion) ~ as.factor(parrot$imj))

##* Do parrotfishes with different feeding strategies have different protrusion distances? What test should we use? ANOVA

feedprot<-aov(log10(parrot$protrusion)~parrot$feeding)

##* How can we access the basic results including the p-value? If we look at the structure we find that the output is incredibly complex, so we can use the function summary to give us the pertinent information

str(feedprot)

summary(feedprot)

plot(log10(parrot$protrusion) ~ parrot$feeding)

##* Does gape and body size covary? What test should we use? CORRELATION as it does not distinguish between independent and dependent variables.

biggape<-cor.test(log10(parrot$body_mass), log10(parrot$gape_distance), alternative="two.sided")

##* If we know the gape of a fish can we predict its protrusion distance? REGRESSION (Y=bX+a)

protgape<-lm(log10(parrot$protrusion)~log10(parrot$gape_distance))

##* This time to see the summary you have to use summary.lm

summary.lm(protgape)

##* Display the result a scatter plot with the regression line using the plot and abline functions, the abline takes the intercept and slope as its arguments or an object of class lm

plot(log10(parrot$protrusion)~log10(parrot$gape_distance))
abline(-0.6451205222662067529, 0.9777115282892123727)
abline(protgape) # another alternative

##* Why might our analysis of gape and protrusion be a problem? There might be a confounding variable: body size may be related to both of these traits. Do fishes with bigger gapes have longer protrusion distances after controling for differences in body mass? What test should we use? MULTIPLE REGRESSION

protgapemass<-lm(log10(parrot$protrusion)~log10(parrot$gape_distance)+log10(parrot$body_mass))

summary.lm(protgapemass)

##* How would you generate a plot to display this result?  

mg<-lm(log10(parrot$gape_distance)~log10(parrot$body_mass))
mp<-lm(log10(parrot$protrusion)~log10(parrot$body_mass))
plot(resid(mp)~resid(mg))
abline(lm(resid(mp)~resid(mg)))

##We can take the residuals from an analysis of mass vs gape and plot them against the residuals from mass vs protrusions.

#===BASIC NON-PARAMETRIC STATISTICS===
## These do no rely on the assumption that the data are drawn from a particularly probability distirbution.

##* Does feeding strategy depend on whether they have an intramandibular joint?What is the appropriate test? CHI-SQUARED TEST

##* Generate the contingency table 

contbl<-table(parrot$imj, parrot$feeding)

feedimj<-chisq.test(contbl, simulate.p.value=TRUE)

# generating p-values through simulation is important when you have small sample sizes

summary(pcparrotsmall)


#==IMPORTANT NOTE==
## We are learning basic statistics today but as we will soon learn when we want to test the hypothesis that mass is related to gestation across a large number of species we are violating one of the assumptions of these methods and that is that each datapoint is independent. Within our data species are related to one another through common ancestry so closely related species are likely to be more similar to each other than distantly related species so they are not independent that is why we use phylogenetic comparative methods.


#=====EXERCISES=====#

## Don't forget to transform the data where necessary and to include factors that could be confounding the relationship

##* Read in the Carnivora Dataset and the new carnivora dataset merge them to generate a single dataset.

c1<-read.table("~/Desktop/Intro2R/CarnivoraDataset.txt", header=T, row.names=1)
c2<-read.table("~/Desktop/Intro2R/CarnivoraDataset2.txt", header=T, row.names=2)

carn<-merge(c1, c2, by.x="row.names", by.y="row.names")

##* 1) Do carnivores with different diets have different geographic ranges sizes after accounting for the influence of body size?

dietmassgr<-aov(log(carn$GR_Area_km2)~factor(carn$Diet)+log(carn$AdultBodyMass_g)) # don't forget diet was read in as an integer but we are wanting it to act as a factor so we have to tell R to do this.
summary(dietmassgr)

##* 2) Is maximum longevity related to gestation length in Otariidae? (controling for body size) 

longest<-lm(log(carn$MaxLongevity_m[carn$Family.x=="Otariidae"])~log(carn$GestationLen_d[carn$Family.x=="Otariidae"])+log(carn$AdultBodyMass_g[carn$Family.x=="Otariidae"]))

##* 3) Do dogs (Canidae) and cats (Felidae) have different sized home ranges? Create a plot to visualize this.

canfelid<-subset(carn, Family.x=="Canidae" | Family.x=="Felidae" )

cfrange<-t.test(log(canfelid$HomeRange_km2)~factor(canfelid$Family.x), na.rm=T)
plot(log(canfelid$HomeRange_km2)~factor(canfelid$Family.x)) # note when we subset, R still recognizes that there are many other possible families even though we only have 2 in our subset, so to reset this we use factor (if we don't it will plot lots of empty boxplots for all the other families).

