#Novemeber 19#

#Today we are going to learn how to do more with tree plotting in R. This will include customizing our trees in ways you might want to do for figures for a paper or a talk. We will also focus on adding trait data to trees that are useful in data exploration and visualization. 

#First of all, we need to ask why we would want to look at trait data on a phylogeny? What types of questions might we be asking? (Character evolution. Ancestral states, transition rates, correlations among characters, covergence, phylogenetic signal, rates)

#This is an empty template that you will fill in during the class, where there is an * it means there is something to do! 

#Load the packages we will be needing today: 
library(ape)
library(geiger)

#What do you all remember about trees in R. What is the object/class of a tree (phylo). What foramt? (Newick or a paranthetical format)

#*Let's read in the tree that we are going to use today; parrotfishtree.nex 
thetree <- read.nexus("parrotfishtree.nex") 

#*Let's plot the tree and add a scale bar.
plot(thetree)
axisPhylo()

#This is a decent looking figure but there are many ways in which you may wish to display a phylogeny for a figure or when exploring your data. We are now going to have some fun manipulating the plotting of this parrotfish tree. Notice that with an understanding of how R deals with edges, tips, and nodes, you have a lot of power to create some pretty useful figures.

#*Recall that when you call plot(tree=phylo_object) then R automatically calls a special plotting function called plot.phylo. To make pretty trees, we want to know some of our options. Let's take a look
?plot.phylo

#*We can see a number of options that autmatically look familar from the plotting you have already done in R, such as cex, font, y.lim, etc. For instance, how would you decrease the font size so that the tip labels do not look so crowded?
plot(thetree, cex=0.5)

#*In addition to the normal plotting options, there are a number of options specific to trees. For instance, you may want to make a circular tree for a figure. You can do this using the 'type' option. Let's try.
plot(thetree, type="fan", cex=0.5)
#*You can play roud with the tree rotation using rotate.tree 
plot(thetree, type="fan", rotate.tree=180)

#*Another useful thing with tree plotting is being able to change the orientation of your tree
plot(thetree, direction="upwards")


#*Perhaps you want to add photos to a figure later instead of having the written tip labels identify your tips. You can remove tips labels using show.tip.label
plot(thetree, show.tip.label=FALSE, type="fan")


#*We can change the color of our tip labels using tip.color and the color of the branches using edge.color
plot(thetree, tip.color="blue", edge.color="purple")

#*Already we can see that R gives us a lot of power to manipulate trees with a few simple plotting commands, but let's try to do something a little more complicated. Say that you specifically want to draw attention to a clade on a phylogeny. Maybe we want to mark the most recent ancestor of the Scarus clade. We can do this manually by plotting the phylogeny, showing the tip labels, and visually picking out the correct one, then plotting the tree again with a star on the correct node.

plot(thetree)
nodelabels() 
#the answer is 51

#*Or we can pick it out by hand with identify 
plot(thetree)
identify(thetree)

#*Plot a star on that node with pch=8, lwd=5, cex=3
plot(thetree)
nodelabels(node=51, pch=8, lwd=5, cex=3, col="purple")


#This works fine on this small tree, but gets much harder when you are dealing with large trees with lots of taxa. Plus, node labels can overlap or be hard to see when you have short branches. Anyway, we will do this in an automated and more elegant way using the mrca() function.

#*To figure out how mrca() works, let's try it and take a look at the first few rows of output 
head(mrca(thetree))

#*Now we can select two taxa to pick the MRCA of the Scarus clade
scarusNode <- mrca(thetree)["Scarus_spinus","Scarus_psittacus"]
plot(thetree)
nodelabels(node=scarusNode, pch=8, lwd=5, cex=3, col="purple")

#*You can save all the taxa (tip labels) in the Scarus clade using the tips() command and the node we just found
clade1 <- tips(thetree, scarusNode)

#*Instead of marking the MRCA, we can "paint" the clade on out phylogney with a little bit of code. First we need to find the edges to paint using which.edge()
scarus_edges <- which.edge(thetree, clade1) 

#*We want to make a vector with the colors to assign to all edges in the tree. It seems easiest to first make a vector with black for all edges. Then we can assign the color purple to the edges of the Scarus clade
mycols <- rep("black", dim(thetree$edge)[1])
mycols[scarus_edges] <- "purple"
plot(thetree, edge.width=4, edge.color=mycols)

#######################################################
#*Finally, lets start visualizing some data on our trees! Read in the data we will be using today: SparisomaFins.txt  (it has column names and row names)
thedata <- read.table("ScarusSparisomaFins.txt", header=T, row.names=1)   

#Do our dataset and phylogeny have exactly the same taxa represented?
thetree
dim(thedata)
name.check(thetree, thedata)

#*We want to sort and match our data using treedata() like Sam showed you last week.  
trdat <- treedata(thetree, thedata, sort=TRUE)

#*Let's save the pruned/sorted tree and dataset. Recall that trdat returns a list with a phylogeny ($phy) and a datafile ($data) that have been sorted to match and have the taxa found only in both of your original files
pruned <- trdat$phy
lessdata <- data.frame(trdat$data)

#*Check that they are sorted and matching
lessdata
pruned$tip.label
name.check(pruned, lessdata)

#*It is useful at this point to rescale our tree to make plotting easier using rescale. We want to make the total depth of the tree equal to 1. 
scaletr <- rescale(pruned, "depth", 1)
plot(scaletr)
axisPhylo()

#*To figure out a bit how the plotting window is arranged try plotting some points onto the phylogeny using points(x=, y=, pch=16)
points(x=1, y=1, pch=16, col="red")
points(x=1, y=23, pch=16, col="blue")
points(x=0.5, y=1, pch= 16, col="black")
points(x=1.1, y=1, pch= 16, col="green")

#Okay so let's take a look at how a discrete character is distributed across our taxa on the phylogeny. One way to do this is to plot circles at the tips that are colored by habitat (0=white, 1=red). 

#*First we want to plot the tree so that it only takes up half the plotting window. We also want to offset tip labels so that we have room to place our squares at the tips 
plot(scaletr, cex=0.5, label.offset=0.3, x.lim=c(0,3), y.lim=c(0,25))
axisPhylo()

#*We're going to make a new matrix to store the colors will use to label our tip taxa
habitatLabel <- character(length(scaletr$tip.label)) 
habitatLabel[lessdata$habitat==0] <- "white"
habitatLabel[lessdata$habitat==1] <- "red"
habitatLabel

#*Now we will add the habitat to the tips using tiplabels(). There are other ways to place your circles (such as with points()), but I like to use tiplabels() since it automatically matches up your tips with the appropriate circle. 
tiplabels(tip=c(1:length(scaletr$tip.label)), pch=21, bg=habitatLabel, cex=1.5, adj=0.5)
text(x=1, y=24, labels="Habitat", cex=0.5, font=2)

#*Now let's take a look at how a continuous character is distributed across our taxa on the phylogeny. One way to do this is to plot circles at the tips that are scaled in size by the value of the trait. We will use the tiplabels() command in a similar fashion to the habitatLabels, however we first need to do a couple things to make R happy...
circles <- as.numeric(as.character(lessdata$fin_angle)) #work around for factors 
circles <- circles/50
tiplabels(tip=c(1:length(scaletr$tip.label)), pch=21, bg="blue", cex=circles, adj=0.7)
text(x=1.2, y=24, labels="Angle", cex=0.5, font=2)

#*Now look at your plot with the habitat and fin angle circles. What hypotheses about character evolution might you be interested in asking based on this visualization?

#Assignment 1-- Paint the branches in the Sparisoma clade green
clade2<- tips(thetree, mrca(thetree)["Sparisoma_radians","Sparisoma_atomarium"])
spari_edges <- which.edge(thetree, clade2)
mycols[spari_edges] <- "limegreen"
plot(thetree, edge.width=4, edge.color=mycols)

#Assignment 2--- Add Ecology label
ecologyLabel <- character(length(scaletr$tip.label)) #We're going to make a new matrix to store the colors will use to label our tip taxa
ecologyLabel[lessdata$ecology ==0] <- "white"
ecologyLabel[lessdata$ecology==1] <- "green"
ecologyLabel
tiplabels(tip=c(1:length(scaletr$tip.label)), pch=21, bg=ecologyLabel, cex=1.5, adj=0.6)
text(x=1.1, y=25, labels="Ecology", cex=0.5, font=2)




