# Introduction to R Day 7: Introduction to phylogenetic comparative methods (PCM)

## Why are phylogenies important when we are analysing interspecific patterns? 

# This is an empty template that you will fill in during the class, where there is an * it means there is something to do! 

##* Open R and if you haven't already done so, set your working directory to your Intro2R folder and load the ape and geiger packages in R. 

library(geiger)

##* Read in the CetaceaLH.txt dataset and the CetaceaPhy.nex and match them 

cetdat<-read.table("~/Desktop/Intro2R/CetaceaLH.txt", header=T)

cetphy<-read.nexus("~/Desktop/Intro2R/CetaceaPhy.nex")

wib<-treedata(cetphy, cetdat, sort=TRUE)

tr<-wib[[1]]
dat<-wib[[2]]

##* Test for a relationship between gestation length and mass by running a simple linear regression and plotting log10 body mass vs log10 gestation length adding the regression line to it 

LMbmgl<-lm(log(dat[,3], 10)~log(dat[,2], 10))
summary.lm(LMbmgl)

plot(log(dat[,2], 10), log(dat[,3], 10), xlab="Log10 Body Mass", ylab="Log10 Gestation Length")
abline(LMbmgl)

##* What would we conclude from these results?  That size and gestation length is only very weakly related within cetaceans: size only explains ~5% of the variance in gestation length. However phylogeny may be driving some of this pattern.  

##* We can visualize how phylogeny might be influencing the relationship between gestation and mass by tracing the phylogeny onto the plot, so you can look for phylogenetic clustering etc. This is a known as phylomorphospace and is available in the phytools package. 

library(phytools)

##* Make a matrix of the two traits without any missing data and prune the tree to match and then plot the phylomorphospace

bmgl<-na.omit(cbind(log(dat[,2], 10), log(dat[,3], 10)))

bmglwib<-treedata(tr, bmgl)

phylomorphospace(bmglwib[[1]], bmglwib[[2]], label="off")

##* What might you conclude from this plot?  All of the really big whales belong to a single clade - they are all baleen whales and they don't have very long gestation lengths compared to the other group (toothed whales). Within toothed whales we can also see evidence of a much steeper slope between mass and gestation length. So it looks like phylogeny may actually be masking the relationship between size and gestation length within cetaceans. The first phylogenetic comparative method ever developed was to deal with the issue of looking for a relationship between traits while taking into account phylogeny and is known as independent contrasts (Felsenstein 1985). Independent contrasts and many other phylogenetic comparative methods (PCM) are based on a model of evolution known as Brownian motion. A model of evolution is necessary as it describes how the trait is expected to evolve upon the phylogeny. So before we learn specific methods of how to analyze data using PCM we first need to understand Brownian motion.


##===BROWNIAN MOTION ===

# What is Brownian motion?  At each instant in time the state of a character can increase or decrease and the direction and magnitude of these changes are independent of current or past states (no trends etc.).  The total displacement of a character after time t is drawn from a normal distribtion with a mean of zero (i.e. net change in trait is 0) and a variance proportional to t. 

##* First generate the displacements of the trait using the rnorm function to generate a normal distribution with a mean of zero for 100 units of time.

displace<-rnorm(100, mean=0, sd=1)
plot(displace)

##* The sum of these displacements represents the trajectory of the trait through time we can calculate the cumulative summation of these displacements using the cumsum function in R

x<-cumsum(displace)
plot(x, type="l", xlab="Time", ylab="Trait Value")


##* Now create a plot with 50 separate runs of Brownian motion. You can imagine these as 50 independent species (sharing no common ancestry). For this we will need to construct a very simple for loop. A for loop allows the same piece of code to be repeatedly executed i.e. it iterates the code. So if you want to run something 50 times you write for(i in 1:50) which means it will start with i=1, go to i=2 all the way to 50. 

plot(cumsum(rnorm(100, mean=0, sd=1)), type="l", ylim=c(-30,30))
for(i in 1:49) lines(cumsum(rnorm(100, mean=0, sd=1))) 

# Here we can see variance in the trait is proportional to time. The further along the x axis we go the greater the variance on the y axis. If you think about sister-species some diverged very recently and others many millenia ago, the longer time they have had to evolve independently the more different the traits of the sister species are likely to be. 

##* Obviously the above simulations were assuming the species were independent but we know that they are not, they share evolutionary history and this affects the evolution of their traits. Think about a 3 species tree:

tree<-read.tree(text="((grizzlybear:1, polarbear:1):4, spectacledbear:5);")
plot(tree)
axisPhylo()

# Looking at this tree we see that grizzly bears and polar bears share about 4 million years of history during which they all follow the same BM trajectory after which grizzly and polar bears split with both lineages starting on a new BM trajectory - we can simulate this. We need the branch length information: the longer the branch the greater the expected variance - with this we determine the standard deviation of the normal distribution. The total displacement of a character after time v is drawn from a normal distribtion with a mean of zero (i.e. net change in trait is 0) and a standard deviation = sqrt(rate*branchlength). The faster the rate the greater the variance as well. 

# Lets assume we have a root state of 1 and a rate of 1

# Start with the first branch from the root node 4 to 5 and estimate the trait value at node 5 and use the branch lengths to estimate the displacement.

x<-rnorm(1, mean=0, sd=sqrt(1*tree$edge.length[1]))

# You obviously need to convert this displacement to a trajectory, so if the root state is 1 then you add the displacement to the root state to get the value at the node.

node5est<-1+x


## === INDEPENDENT CONTRASTS ===

## Independent contrasts divide up the phylogeny into "independent" contrasts so we can identify a relationship between the traits independent of phylogeny. If we have a phylogeny of 4 species: A & B are sister taxa as are C & D, then there are 3 independent comparisons, one between A & B, one between C & D and finally, one between A&B and C&D the contrasts are calculated as the difference between the trait values from these comparisons. Brownian motion is used to standardize the contrast, as sister-taxa that have had more time to evolve independently are expected to have larger contrasts, so the contrasts are divided by the sqrt of the of the sum of the branch lengths between (standard deviation of the contrast). Also contrasts deeper in the phylogeny are "corrected", as deeper within the phylogeny our methods of estimating the trait values becomes more uncertain as we are estimating the trait value thus we downweight the internal contrasts. This was explained on the black-board in more detail - hopefully you took notes!

##* Using the pic function in ape calculate the contrasts for body size and gestation length and plot them against one another as well as run the regression on the contrasts. Because the sign of the contrasts is arbitrary the regression must go through the origin which you can do in R using lm(equation, -1) 

picbm<-pic(bmglwib[[2]][,1], bmglwib[[1]], scaled=T, var.contrasts=T) # we want them to be scaled by the branch length so scaled=T which is the default, we also want them to give us the standard deviation i.e. sqrt of the branch length with var.contrasts=T

picgl<-pic(bmglwib[[2]][,2], bmglwib[[1]], scaled=T, var.contrasts=T) 

piclmbmgl<-lm(picgl[,1] ~ picbm[,1] -1)
summary.lm(piclmbmgl) # look at the amount of variance explained now we have correctly included phylogeny it is significantly more than we saw when we did not include phylogeny in our calculations.
plot(picbm[,1], picgl[,1])
abline(piclmbmgl)

## This is just the tip of the iceberg, we will continue to explore Brownian motion and independent contrasts next week!


##=====EXERCISES=====#
# This is going to be short today as we spent a lot of time going over Brownian motion and independent contrasts.

##* 1) Create a plot with 20 separate runs of Brownian motion assuming independent evolution i.e. ignore phylogeny, 10 evolving at a slower rate than the other 10 (i.e. change the SD of the normal distribution which is related to time & rate but we are ignoring time because we are assuming independent evolution therefore SD = rate).

plot(cumsum(rnorm(100)), type="l", ylim=c(-80,80), ylab="Trait", col="blue")
for(i in 1:9) lines(cumsum(rnorm(100)), col="blue") #slower
for(i in 1:10) lines(cumsum(rnorm(100, sd=5)), col="green") #faster

##* 2) Calculate the relationship between mass and maximum longevity using independent contrasts. 

bmml<-na.omit(cbind(log(dat[,2], 10), log(dat[,4], 10)))
bmmlwib<-treedata(tr, bmml)
picbm<-pic(bmmlwib[[2]][,1], bmmlwib[[1]], scaled=T, var.contrasts=T) 
picml<-pic(bmmlwib[[2]][,2], bmmlwib[[1]], scaled=T, var.contrasts=T) 
piclmbmml<-lm(picml[,1] ~ picbm[,1] -1)
summary.lm(piclmbmml) 
plot(picbm[,1], picml[,1])
abline(piclmbmml)
