unscaletrait1pic# Introduction to R Day 8: A deeper look at Brownian motion, independent contrasts and phylogenetic comparative methods (PCM)

# This is an empty template that you will fill in during the class, where there is an * it means there is something to do! 

##* Open R and if you haven't already done so, set your working directory to your Intro2R folder and load the ape and geiger packages in R. 

library(ape)

##* Read in the very simple simulated dataset and tree Sim10spdat.txt and Sim10sptree.nex

dat<-read.table("Sim10spdat.txt")
tr<-read.nexus("Sim10sptree.nex")

##* plot the tree - which two species should have the most similar trait values and why?  


##* How many independent contrasts are there in the tree? Remember independent contrasts is a method used to look for a relationship between traits while taking into account the important fact that each species is not independent, it shares evolutionary history with its closely related species. 

tr # there are 10 species so the number of contrasts is always one fewer = 9.

##* Calculate the scaled and unscaled independent contrasts of trait 1 and 2 using the pic command in ape add the variance of the constrasts to your raw pic

unscaletrait2pic<-pic(dat$trait2, tr, scaled=FALSE, var.contrasts=T)
scaletrait2pic<-pic(dat$trait2, tr, scaled=TRUE)

unscaletrait1pic<-pic(dat$trait1, tr, scaled=FALSE, var.contrasts=T)
scaletrait1pic<-pic(dat$trait1, tr, scaled=TRUE)

##* Now create 3 plots 1) raw data trait 2 vs trait 1 2) unscaled constrasts trait 2 vs 1 and 3) scaled contrasts trait 2 vs 1. Remember that there 2 columns in the unscaled data we want to analyze the contrasts in the first column

par(mfrow=c(1,3)) # mfrow allows you to specify the number of rows and columns of the plots, so we are going to be plotting 3 graphs next to one another on the same row with this command. 
plot(dat$trait2, dat$trait1)
plot(unscaletrait2pic[,1], unscaletrait1pic[,1])
plot(scaletrait2pic, scaletrait1pic)

##* Estimate the stength of the relationship for these 3 datasets, remember we have to force the regression line through the origin when we are analyzing contrasts 

lmraw<-lm(dat$trait1~dat$trait2)
lmunscale<-lm(unscaletrait1pic[,1] ~ unscaletrait2pic[,1]-1)
lmscale<-lm(scaletrait1pic ~ scaletrait2pic-1)

summary(lmraw)
summary(lmunscale)
summary(lmscale)

##What is going on here? Which of these results should we trust? 

## First think about the difference between the scaled and unscaled contrasts (think back to our discussions last week). What are the unscaled contrasts? Plot the tree with the contrasts of trait 1 at the nodes and the values of trait 1 at the tips using the techniques Sarah Longo introduced while I was away.

plot(tr, show.tip.label=FALSE)
tiplabels(dat$trait1)
nodelabels(scaletrait1pic)

## Look at the contrast between species B & C you can see it is simply the trait value of B - C. 

##Remember scaling or standardizing the contrasts is critical as it controls for the differences in branch length i.e. our expectation that more closely related species are going to be more similar to one another than more distantly related species. It does this by using a Brownian motion model whereby the expected variance is proportional to time. Specifically the raw constrast is divided by the sqrt of the sum of the branch (+ the correction factor to take into account the fact that deeper in the phylogeny our estimates of the trait values are more uncertain)). The unstandardized contrasts only generate evolutionary independent comparisons but don't control for time and the expectation that the longer taxa have been evolving the independently the greater the expected difference between their phenotypes. 

##* Try scaling the unscaled contrast of trait 2, you can do this by taking the square-root of the expected variance (sum of the branch (+ the correction factor to take into account the fact that deeper in the phylogeny our estimates of the trait values are more uncertain)). The expected variance was output into column 2 when we used var.contrasts=T. These values should be identical to the scaled contrast  - is it?

unscaletrait2pic[,1]/sqrt(unscaletrait2pic[,2])
scaletrait2pic

## Checking the assumptions of Brownian motion
# Again it is important to realize we are assuming a model of Brownian motion, how can we check if this an appropriate assumption? 

##* If we have correctly standardized our contrasts under Brownian motion there should no longer be any relationship between the absolute value (i.e. ignoring the sign) of the standardized contrast and the expected variance i.e sum of the branch (+ the correction factor to take into account the fact that deeper in the phylogeny our estimates of the trait values are more uncertain). 

plot(sqrt(unscaletrait2pic[,2]), abs(scaletrait2pic)) # abs is the function to just take the absolute value
cor.test(sqrt(unscaletrait2pic[,2]), abs(scaletrait2pic))
plot(sqrt(unscaletrait1pic[,2]), abs(scaletrait1pic))
cor.test(sqrt(unscaletrait1pic[,2]), abs(scaletrait1pic))

##* Moreover, under Brownian motion the absolute value of standardized contrasts should not be related to the age of the node. We can access this using the function branching.times in the ape package for trait 1 and trait 2

plot(branching.times(tr), abs(scaletrait2pic))
cor.test(branching.times(tr), abs(scaletrait2pic))

plot(branching.times(tr), abs(scaletrait1pic))
cor.test(branching.times(tr), abs(scaletrait1pic))

# Why does Brownian motion specifically predict that the absolute value of the standardize contrast should not be related to age? Well what would it mean if the absolute contrasts were much higher closer to the root?  That more variance was generated early in the evolutionary history of the clade. Think about the Brownian motion model that we simulated last week under it variance is proportional to time & what else? The rate. The faster the rate the greater the variance. So what may have generated greater variance close to the root - faster rates! But a strict Brownian model means constant rates so if there is a positive or negative relationship between the age of the node and the absolute contrast it suggests that the rate has changed over time. 

## Now we have confirmed that our two traits fit a Brownian motion model, should we choose to use the raw data or the standardized independent contrasts to analyze our data?

##* Last week we used the phylomorphospace to visualize the potential influence of phylogeny. Plot trait 1 vs 2  using a phylomorphospace what do you conclude?

phylomorphospace(tr, cbind(dat[1], dat[2]))

# It looks like closely related species share similar morphologies so we might expect to have to take into account phylogeny.

# However, strictly speaking we care if the residuals from the regression are phylogenetically patterned, only then we need to take into account phylogeny. So it is important to realize it is not the traits themselves that are important here but the residuals! Just like violations of normality for regular regressions focus the normality of the errors i.e. the residuals. There is a variety of methods used for estimating the degree of phylogenetic signal in a trait but they are all based on Brownian motion as Brownian motion leads to traits being more similar among closely related species. 


## PHYLOGENETIC SIGNAL USING BLOMBERG'S K

# The K statistic is the ratio of the observed statistic to that expected under Brownian motion thus K= 1 suggests perfect Brownian motion, K < 1 weaker signal than expected under BM or stronger if K > 1 (i.e. more closely related species are even more similar in their traits than you expect from Brownian motion). The actual statistic is the mean squared error of the tip data divided by the phylogenetically corrected mean squared error of the data. This is implemented in the Kcalc function in the picante package. 

library(picante)

##* Calculate the residuals from the linear regression of the raw trait data of trait 2 vs trait 1 and estimate K. 
foo1<-resid(lmraw)
names(foo1)<-row.names(dat)

Kcalc(foo1, tr)

##* It looks like phylogenetic comparative methods are justified. Compare the regressions of the raw data and the standardize contrasts that we generated at the start of class. What can you conclude has happened? 

summary(lmraw)
summary(lmscale)

##=====EXERCISES=====#

#1) Calculate the standardized independent contrasts of trait 3 & 4 and check that Brownian motion is an appropriate model

trait3pic<-pic(dat$trait3, tr, var.contrasts=T)

trait4pic<-pic(dat$trait4, tr, var.contrasts=T)

# test absolute value of the standardized contrast vs expected variance
par(mfrow=c(1,2))
plot(sqrt(trait3pic[,2]), abs(trait3pic[,1])) # abs is the function to just take the absolute value
cor.test(sqrt(trait3pic[,2]), abs(trait3[pic,1]))
plot(sqrt(trait4pic[,2]), abs(trait4pic[,1])) # abs is the function to just take the absolute value
cor.test(sqrt(trait4[,2]), abs(trait4[,1]))

# test for rate changes over time
plot(branching.times(tr), abs(trait3pic[,1]))
cor.test(branching.times(tr), abs(trait3pic[,1]))
plot(branching.times(tr), abs(trait4pic[,1]))
cor.test(branching.times(tr), abs(trait4pic[,1]))

#2) Run a regression on the raw species values and the independent contrasts to determine if trait 2 and 3 are related. Then decide which you prefer and why.

lmphy<-lm(trait3[,1] ~ scaletrait2pic-1)
summary(lmphy)

lmraw<-lm(dat$trait3 ~ dat$trait2)
summary(lmraw)

lmfoo<-lm(dat$trait3~dat$trait2)
lmresid<-resid(lmfoo)
names(lmresid)<-row.names(dat)

Kcalc(lmresid, tr)
phylomorphospace(tr, cbind(dat$trait3, dat$trait2))

# Although it appears to not be perfectly Brownian as K<1, there is obviously an influence of phylogeny upon data so I would recomend using the contrasts. If we look at the R^2 the amount of variance explained by the contrasts is lower than the raw data suggesting that phylogeny does explain some of the variance. 


# 3) What model of trait evolution best describes trait 5?

# we can estimate the phylogenetic signal within the trait using K, K=1 it is Brownian

Kcalc(dat[5], tr) # it is not Brownian!

trait5pic<-pic(dat$trait5, tr, var.contrasts=T)

plot(branching.times(tr), abs(trait5pic[,1]))
cor.test(branching.times(tr), abs(trait5pic[,1]))

# It is not Brownian motion that is for sure! 1) closely related species have much more similar trait values than expected under Brownian. 2) There is a relationship between node age and the absolute value of the contrast, such that contrasts deeper within the tree show greater variance. Thus we can conclud that one model that may explain these results is one of rates decreasing over time.  