# Introduction to R Day 9: Principal Components Analysis

# This is an empty template that you will fill in during the class, where there is an * it means there is something to do! 

## Why do we use a principal components analysis (PCA)?  In morphological datasets we often use it to reduce dimensionality of our data if we have lots of measurements that may be all related to one another as it creates axes that are orthogonal combinations of individual traits. It allows us to to identify patterns within in complex datasets. The principal components are the main axes of variation within the data and there is as many axes as there are dimensions within your dataset. These techniques are useful whenever you have large datasets. 

## What is principal components analysis? We identify the directions (components) that maximize the variance in our dataset. If you imagine two traits which form a cloud of points the shape of a football the primary axis of variation is between the two ends of the football and the second axis must be perpendicular to it. The eigenvectors describe the direction of the axis and the eigenvalues tell you how much variance there is in the data in that direction. The eigenvector with the highest eigenvalue is therefore the principal component which is between the two ends of the football. The number of eigenvectors in the dataset is the total number of dimensions i.e. the number of traits. 

# Principal components analysis is subject to the same restrictions as regression, in particular multivariate normality and they are also highly influenced by outliers. 


##* Open R and if you haven't already done so, set your working directory to your Intro2R folder. Read in the parrotfishecology.txt and the parrotfishesRawMorphology.txt dataset.

parrotdat<-read.table("~/Dropbox/Intro2R/Datasets/parrotfishesRawMorphology.txt", header=T, row.names=1)

parrotecol<-read.table("~/Dropbox/Intro2R/Datasets/parrotfishecology.txt", header=T, row.names=1)

##* Generate a new subset of the data comprising of gape, protrusion, adductor mass, sternohyoideus and levator posterior mass all log10

names(parrotdat)

newdat<-log(parrotdat[,c(3:7)], 10)

##* If the data are on different scales we need to run the principal components analysis on the correlation matrix of the traits rather than the covariance - this is sometimes called scaling. If we don't scale the data the variables with the greatest variance will dominate the dataet and usually in morphological datasets are data are measured in different units and need to be scaled. 

pcamorph<-prcomp(newdat, scale=T)
summary(pcamorph)

##* You want to know what traits the principal axes are comprised of and how much of the variance in the datasets is explained by each axis. You can use summary to access the variation explained by each principal component.

pcamorph$rotation
pcamorph$x

## The loadings or as referred to by prcomp the rotations express the relationship between the original traits and the new principal components axis. It allows you to interpret the influence that each trait has on each axis. The higher the absolute value the stronger the contribution of that trait to that particular axis and the sign tells you whether an increase on the axis is associated with larger (+ve) or smaller (-ve) values of the trait.


##* For most morphological datasets the first axis will be related to the overall size of the organism unless you have done something to remove the influence of size. This is true for this dataset, PC1 the loadings for each trait all have the same sign (-ve) and are roughly the same size and from the summary you can see it explains 86% of the variation in the dataset. As PC1 increases size descreases - is that right? We can plot the relationship between PC1 and standard length in our original dataset to investigate. Where each species falls on the principal components axis is given by the scores,  which are in the 5th element of the output list. Because the species in the PCA and our original dataset are in the same order (you can do cbind(row.names(parrotdat), row.names(pcamorph$x)) to check) we can plot them against one another without sorting etc:

plot(log(parrotdat[,1], 10), pcamorph$x[,1], xlab="log length", ylab="PC1")

##* But what about PC2? it explains 10% of the variation, one variable contributes strongly to this axis and it is protrusion, the larger the value of PC2 the smaller the jaw protrusion but there is also some effect of gape, levator muscle and the sternohyoideus and they are all positive so as PC2 increases so does gape etc. So if we plot protrusion against PC2 we get a strong -ve relationship but there is more variance than there was when we plotted PC1 vs standard length because gape etc. also influence PC2

plot(log(parrotdat[,4], 10), pcamorph$x[,2], xlab="log protrusion", ylab="PC2")

##* We can plot the principal components axes against one another to illustrate the morphospace described by these new variables. What can you infer from this plot?

plot(pcamorph$x[,1], pcamorph$x[,2], xlab="PC1", ylab="PC2")

##* The very largest species also seem to have the largest values on PC2, which means they have small jaw protrusions but larger gape etc. Now you would probably be interested in seeing if this relates to specific diets or feeding ecologies we can do this by colouring the species in the morphospace to represent the ecology of interest. As the parrotfish ecology dataset is in the same species order as the morphological data we don't need to sort it. Can you remember how to create a vector of colours to represent a categorical trait? It was covered in Day 7! Create a vector of colours to represent the feeding strategy

feedcol<-rep("black", length(parrotecol$feeding))
feedcol[parrotecol$feeding=="bite"]<-"orange"
feedcol[parrotecol$feeding=="excavate"]<-"blue"
feedcol[parrotecol$feeding=="excavate/bite"]<-"turquoise"

plot(pcamorph$x[,1], pcamorph$x[,2], xlab="PC1", ylab="PC2", col=feedcol, pch=16)

## There appears to be no clustering of similar ecologies within the morphospace

##* What is the potential problem with this analysis?

#==PHYLOGENETIC PCA==

##* Just like the correlations and regressions we have discussed in the past two weeks PCA can also be influenced by phylogeny, so if we have a large interspecific dataset we may want to calculate the principal components analysis using the phylogeny. Read in the parrotfish tree and match it to the new morphological dataset we created

library(geiger)
library(phytools)

parrottr<-read.nexus("~/Dropbox/Intro2R/Phylogenies/parrotfishtree.nex")

row.names(newdat)[1]<-"Bolbometopon_muricatum"

foo<-treedata(parrottr, newdat, sort=T)


##* Use the phylogenetic PCA in phytools - remember to use the scaling this time you need the argument mode="corr" to specify this. 

phypcamorph<-phyl.pca(foo[[1]], foo[[2]], mode="corr")

##* Once again, you can use summary to access the variation explained by each principal component.
summary(phypcamorph)
summary(pcamorph)
##* The loadings are found in the fourth element of the output and the scores in the second. How does the phylogenetic PCA compare with the ones without including phylogeny? Including the phylogeny means that PC1 and 2 now explain more of the variance and PC1 is still size and protrusion still loads heavily onto PC2 but gape the levator muscle are less important. Don't worry about the signs, they have flipped but they still mean the same thing - the signs are arbitrary between methods or implementations. 

phypcamorph$L

par(mfrow=c(1,2))
plot(pcamorph$x[,1], pcamorph$x[,2], xlab="PC1", ylab="PC2")
plot(phypcamorph$S[,1], phypcamorph$S[,2], xlab="PC1", ylab="PC2")

## Create a phylomorphospace with feeding ecology for PC1 & 2 with the phylogenetic PCA

##* Match the ecological data to the tree and correct the speling mistake of Bolobometopon_muricatum to Bolbometopon_muricatum

row.names(parrotecol)[row.names(parrotecol)=="Bolobometopon_muricatum"]<-"Bolbometopon_muricatum"

foo2<-treedata(parrottr, parrotecol, sort=T)

##* To make a vector of colours for the phylomorphospace to depict ecology we need to provide colours for the internal nodes as well as the tips. We are not interested in the nodes so we are going to colour them grey.

nodecol<-rep("grey", foo2[[1]]$Nnode)
phyfeedcol<-rep("black", length(foo2[[2]][,3]))
phyfeedcol[foo2[[2]][,3]=="bite"]<-"orange"
phyfeedcol[foo2[[2]][,3]=="excavate"]<-"blue"
phyfeedcol[foo2[[2]][,3]=="excavate/bite"]<-"turquoise"
phycol<-c(phyfeedcol, nodecol)
names(phycol)<-1:(length(foo2[[1]]$tip)+foo2[[1]]$Nnode)


## To specify the node colours we have to specify it using the control=list(col.node=) argument in the phylomorphospace function

phylomorphospace(foo2[[1]], cbind(phypcamorph$S[,1], phypcamorph$S[,2]), xlab="PC1", ylab="PC2", pch=16, control=list(col.node=phycol))

# What can you conclude about the influence of ecology on feeding morphology across parrotfishes once you include the influence of phylogeny? Is it any different from the non-phylogenetic analysis in this example?


#=====EXERCISES=====#

## 1) Open the Carnivore dataset, and create a new smaller dataset of just Canidae with data on natural log adult body mass, gestation length, weaning age, inter-birth interval with no missing data

carndat<-read.table("~/Desktop/Intro2R/CarnivoraDataset.txt", header=T, row.names=1)
canidfulldat<-subset(carndat, Family=="Canidae")
caniddatna<-cbind(log(canidfulldat$AdultBodyMass_g), log(canidfulldat$WeaningAge_d), log(canidfulldat$InterbirthInterval_d), log(canidfulldat$GestationLen_d))
row.names(caniddatna)<-row.names(canidfulldat)
caniddat<-na.omit(caniddatna)
colnames(caniddat)<-c("logmass","logweaning", "logibi", 'loggestation')

## 2) Run a phylogenetic PCA on the reproductive characters (don't include body mass), plot PC1 vs PC2 (remember to use the scores in accessed via $S) how do you interpret this plot? What do PC1 and PC2 represent?

carntr<-read.nexus("~/Desktop/Intro2R/carnivoretree.nex")

carnfoo<-treedata(carntr, caniddat, sort=T)

carnphypcamorph<-phyl.pca(carnfoo[[1]], carnfoo[[2]][,2:4], mode="corr")

plot(carnphypcamorph$S[,1],carnphypcamorph$S[,2])

## 3) Decide which principal components axis is most likely to represent a composite measure of size and plot it against body mass, is there a relationship in the expected direction?

plot(carnphypcamorph$S[,3]~carnfoo[[2]][,1]) # I chose PC3 because it is the only axes where all 3 traits load equally and in the same direction, which means it is most likely to be an overall size axis. Because each trait has a negative relationship as PC3 increases weaning age, interbirth interval and gestation go down and because across mammals we generally expect slower life histories with larger species I would expect large values on PC3 to represent small species, thus I would predict a negative relationship between PC3 and body mass and that is what we see.


## 4) Does ln body mass in Canidae evolve under Brownian motion? and if not, can you infer anything about it did evolve? 

canidbm<-na.omit(canidfulldat[2]) # our carnfoo dataset is very much reduced so this will give me all the canid bodysize data that we have

foo3<-treedata(carntr, canidbm, sort=T)

library(picante)
Kcalc(log(foo3[[2]][,1]), foo3[[1]]) # signal appears to be Brownian K=1.06 - you will see warnings about the dataset lacking taxon names so it is assuming the data are in the same order as the tree - this is okay as we sorted them to match when we used the treedata function. 

picbm<-pic(log(foo3[[2]][,1]), foo3[[1]], scaled=T)

plot(branching.times(foo3[[1]]), abs(picbm)) # appears to be no strong pattern which is confirmed by running a linear regression, therefore we can conclude body mass very much evolves as if by Brownian motion. 
lmtest<-lm(abs(picbm)~branching.times(foo3[[1]]))
summary(lmtest)